package org.ow2.bonita.definition.activity;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.jbpm.pvm.Execution;
import org.ow2.bonita.definition.ProcessParameter;
import org.ow2.bonita.definition.ProcessParameter.ParameterType;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.impl.AutomaticBodyImpl;
import org.ow2.bonita.facade.uuid.AutomaticBodyUUID;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.bonita.services.Recorder;
import org.ow2.bonita.services.handlers.FinishedInstanceHandler;
import org.ow2.bonita.util.EnvTool;


public class EndingNodeBehaviour extends AbstractActivity {

  private static final Logger LOG = Logger.getLogger(EndingNodeBehaviour.class.getName());
  
  public EndingNodeBehaviour() {
    super("BonitaEnd", JoinType.XOR, SplitType.AND, ActivityType.route, null);
  }

  @Override
  public ActivityBody getBody(XpdlExecution xpdlExecution, ActivityInstance<ActivityBody> activityInstance) {
    return new AutomaticBodyImpl(new AutomaticBodyUUID(activityInstance.getUUID()));
  }

  @Override
  protected boolean bodyStartAutomatically() {
    return true;
  }

  @Override
  protected boolean executeBusinessLogic(Execution execution) {
    XpdlExecution xpdlExecution =  (XpdlExecution) execution;
    XpdlInstance xpdlInstance = xpdlExecution.getXpdlInstance();
    xpdlInstance.setInstanceState(InstanceState.FINISHED);
    final Recorder recorder = EnvTool.getRecorder();
    recorder.recordBodyEnded(this, xpdlExecution);
    recorder.recordInstanceEnded(xpdlInstance, EnvTool.getUserId());

    XpdlExecution execToSignal = xpdlInstance.getExecutionToSignal();
    if (execToSignal != null) {
      // We are in a subflow, create map of output parameters to signal parent instance
      Map<String, Object> parameterValues = null;
      List<ProcessParameter> processParameters = xpdlExecution.getProcessDefinition().getParameters();
      if (processParameters != null) {
        parameterValues = new HashMap<String, Object>();
        for (ProcessParameter processParameter : processParameters) {
          if (processParameter.getType().equals(ParameterType.OUT)
              ||
              processParameter.getType().equals(ParameterType.INOUT)
          ) {
            String parameterName = processParameter.getName();
            parameterValues.put(parameterName, xpdlExecution.getVariable(parameterName));
          }
        }
      }
      execToSignal.signal(SubFlow.SUBFLOW_SIGNAL, parameterValues);
      xpdlInstance.setExecutionToSignal(null);
    } else {
      // We are in main instance -> finish
      FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
      handler.handleFinishedInstance(xpdlInstance);
    }
    return true;
  }

  @Override
  protected void end(XpdlExecution xpdlExecution) {
    xpdlExecution = (XpdlExecution) xpdlExecution.destroyScope(xpdlExecution.getNode());
    xpdlExecution.end();
    XpdlExecution parent = xpdlExecution.getParent();
    if (parent != null) {
      parent.removeExecution(xpdlExecution);
    }
  }
}
