package org.ow2.bonita.definition.activity;

import java.util.Map;

import org.jbpm.pvm.Execution;
import org.jbpm.pvm.activity.ActivityExecution;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.impl.AutomaticBodyImpl;
import org.ow2.bonita.facade.uuid.AutomaticBodyUUID;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.bonita.services.Recorder;
import org.ow2.bonita.util.EnvTool;

/**
 * Node behaviour for initial node.
 * Initial node is a "virtual" node added by bonita.
 *
 * The default behaviour is a wait state.
 * The execution waits for a signal to really start the instance. This way,
 * the method that created the instance can update some values in the instance
 * (e.g. execution to signal on completion for a subflow instance) before the
 * execution is really started.
 *
 * The instance is recorded as created only after this activity has been signaled.
 *
 * @author Guillaume Porcher
 *
 */
public class InitialNodeBehaviour extends AbstractActivity {

  public InitialNodeBehaviour() {
    super("BonitaInit", JoinType.XOR,
        SplitType.AND,
        ActivityType.route,
        null);
  }

  @Override
  public ActivityBody getBody(XpdlExecution xpdlExecution, ActivityInstance<ActivityBody> activityInstance) {
    return new AutomaticBodyImpl(new AutomaticBodyUUID(activityInstance.getUUID()));
  }

  @Override
  protected boolean bodyStartAutomatically() {
    return true;
  }

  @Override
  public void execute(ActivityExecution execution) {
    execution.waitForSignal();
  }

  @Override
  public void signal(ActivityExecution execution, String signal, Map<String, Object> parameters) {
    final XpdlExecution xpdlExecution = (XpdlExecution) execution;
    final XpdlInstance xpdlInstance = xpdlExecution.getXpdlInstance();
    xpdlInstance.setInstanceState(InstanceState.STARTED);
    final Recorder recorder = EnvTool.getRecorder();
    recorder.recordInstanceStarted(xpdlInstance, EnvTool.getUserId());
    recorder.recordEnterActivity(this, xpdlExecution);
    recorder.recordBodyStarted(this, xpdlExecution);
    recorder.recordBodyEnded(this, xpdlExecution);
    executeSplit(execution, false);
  }

  @Override
  protected boolean executeBusinessLogic(Execution execution) {
    return false;
  }

}
