/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.io.IOException;
import java.util.logging.Logger;

import org.jbpm.pvm.env.Environment;
import org.ow2.bonita.runtime.ClassDataLoader;
import org.ow2.bonita.runtime.Repository;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.Command;
import org.ow2.novabpm.util.Misc;

public class SerializedCommand<T> implements Command<T> {

  private static final long serialVersionUID = -3780999471259012607L;
  private byte[] serializedCommand;
  private String className;
  private static final Logger LOG = Logger.getLogger(SerializedCommand.class.getName());

  public SerializedCommand(final Command< ? > command) {
    try {
      this.className = command.getClass().getName();
      this.serializedCommand = Misc.serialize(command);
    } catch (final IOException e) {
      throw new BonitaRuntimeException(e);
    } catch (final ClassNotFoundException e) {
      throw new BonitaRuntimeException(e);
    }
  }

  @SuppressWarnings("unchecked")
  public T execute(final Environment environment) throws Exception {
    final ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
    ClassLoader classLoader = contextClassLoader;
    // check if the class is available in current classloader
    try {
      Class.forName(this.className);
    } catch (final ClassNotFoundException e) {
      // the class is not available in current classloader. Look for this class
      // in global class loaders
      final Repository repository = EnvTool.getRepository();
      classLoader = ClassDataLoader.getGlobalClassLoader(repository, this.className);
    }
    if (classLoader == null) {
      throw new BonitaRuntimeException("Class: " + this.className + ", can't be found.");
    }

    T result = null;
    try {
      // set classloader
      Thread.currentThread().setContextClassLoader(classLoader);
      // deserialize command
      final Command<T> command = (Command<T>) Misc.deserialize(this.serializedCommand);
      // execute command
      result = command.execute(environment);
    } finally {
      // set classloader back...
      Thread.currentThread().setContextClassLoader(contextClassLoader);
    }
    return result;
  }
}
