/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb.ejb2;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.rmi.RemoteException;

import org.ow2.bonita.facade.exception.BonitaInternalException;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.EJBUtil;

// This class is only there to wrap try{...} catch(RemoteException e){...}
public class EJB2ClientAPIInterceptor implements InvocationHandler {

  private Object api;

  public EJB2ClientAPIInterceptor(Object api) {
    this.api = api;
  }

  public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {

    try {
      final Method m = api.getClass().getMethod(method.getName(), method.getParameterTypes());
      return m.invoke(api, args);
    } catch (Exception e) {
      if (e instanceof InvocationTargetException) {
        Throwable invocationExceptionCause = e.getCause();
        if (invocationExceptionCause instanceof BonitaException) {
          //normally we should never be here as we are in remote mode
          throw invocationExceptionCause;
        } else if (invocationExceptionCause instanceof RemoteException) {
          RemoteException remoteException = (RemoteException) invocationExceptionCause;
          Throwable remoteCause = getRemoteCause(remoteException);
          EJBUtil.manageInvokeExceptionCause(method, remoteCause);
        }
      }
      throw new BonitaInternalException("Ouch! Unexpected error: " + e, e);
    }
  }

  private Throwable getRemoteCause(RemoteException e) {
    Throwable t = e;
    while (t instanceof RemoteException) {
      t = t.getCause();
    }
    return t;
  }

}

