package org.ow2.bonita.facade.ejb.ejb2.internal;

import java.util.Set;

import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ParticipantDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.ActivityDefNotFoundException;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.DataFieldNotFoundException;
import org.ow2.bonita.facade.exception.PackageNotFoundException;
import org.ow2.bonita.facade.exception.ParticipantNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ParticipantDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.EJBUtil;

public class EJB2InternalQueryDefinitionAPI implements QueryDefinitionAPI {

  protected QueryDefinitionAPI getAPI() {
    return EJBUtil.getEJB2APIAccessor().getQueryDefinitionAPI();
  }

  public PackageDefinition getPackage(String packageId, String version) throws PackageNotFoundException {
    return getAPI().getPackage(packageId, version);
  }
  public Set<ProcessDefinition> getProcesses(String processId, String version) {
    return getAPI().getProcesses(processId, version);
  }
  public PackageDefinition getLastPackage(String packageId) throws PackageNotFoundException {
    return getAPI().getLastPackage(packageId);
  }

  public ProcessDefinition getLastProcess(String packageId, String processId) throws ProcessNotFoundException {
    return getAPI().getLastProcess(packageId, processId);
  }

  public PackageDefinition getPackage(PackageDefinitionUUID packageDefinitionUUID) throws PackageNotFoundException {
    return getAPI().getPackage(packageDefinitionUUID);
  }

  public Set<ProcessDefinition> getPackageProcesses(PackageDefinitionUUID packageDefinitionUUID) throws PackageNotFoundException {
    return getAPI().getPackageProcesses(packageDefinitionUUID);
  }

  public Set<PackageDefinition> getPackages() {
    return getAPI().getPackages();
  }

  public Set<PackageDefinition> getPackages(String packageId) {
    return getAPI().getPackages(packageId);
  }

  public Set<PackageDefinition> getPackages(PackageDefinition.PackageState state) {
    return getAPI().getPackages(state);
  }

  public Set<PackageDefinition> getPackages(String packageId,
      PackageDefinition.PackageState state) throws PackageNotFoundException {
    return getAPI().getPackages(packageId, state);
  }

  public ProcessDefinition getProcess(ProcessDefinitionUUID processDefinitionUUID) throws ProcessNotFoundException {
    return getAPI().getProcess(processDefinitionUUID);
  }

  public Set<ActivityDefinition> getProcessActivities(ProcessDefinitionUUID
      processDefinitionUUID) throws ProcessNotFoundException {
    return getAPI().getProcessActivities(processDefinitionUUID);
  }

  public ActivityDefinition getProcessActivity(
      ProcessDefinitionUUID processDefinitionUUID, String activityId) throws ProcessNotFoundException, ActivityNotFoundException {
    return getAPI().getProcessActivity(processDefinitionUUID, activityId);
  }

  public ActivityDefinitionUUID getProcessActivityId(
      ProcessDefinitionUUID processDefinitionUUID, String activityName) throws ProcessNotFoundException {
    return getAPI().getProcessActivityId(processDefinitionUUID, activityName);
  }

  public ParticipantDefinition getProcessParticipant(
      ProcessDefinitionUUID processDefinitionUUID,
      String participantId) throws ProcessNotFoundException, ParticipantNotFoundException {
    return getAPI().getProcessParticipant(processDefinitionUUID, participantId);
  }

  public ParticipantDefinitionUUID getProcessParticipantId(
      ProcessDefinitionUUID processDefinitionUUID, String participantName) throws ProcessNotFoundException {
    return getAPI().getProcessParticipantId(processDefinitionUUID, participantName);
  }

  public Set<ParticipantDefinition> getProcessParticipants(
      ProcessDefinitionUUID processDefinitionUUID) throws ProcessNotFoundException {
    return getAPI().getProcessParticipants(processDefinitionUUID);
  }

  public Set<ProcessDefinition> getProcesses() {
    return getAPI().getProcesses();
  }

  public Set<ProcessDefinition> getPackageProcesses(String packageId, String processId) {
    return getAPI().getPackageProcesses(packageId, processId);
  }
  
  public Set<ProcessDefinition> getProcesses(String processId) {
    return getAPI().getProcesses(processId);
  }

  public Set<ProcessDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
    return getAPI().getProcesses(processState);
  }

  public Set<ProcessDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
    return getAPI().getProcesses(processId, processState);
  }

  public ProcessDefinition getPackageProcess(
      PackageDefinitionUUID packageDefinitionUUID, String processId) throws PackageNotFoundException, ProcessNotFoundException {
    return getAPI().getPackageProcess(packageDefinitionUUID, processId);
  }
  
  public ProcessDefinition getProcess(String packageId, String processId, String version) throws ProcessNotFoundException {
    return getAPI().getProcess(packageId, processId, version);
  }

  public DataFieldDefinition getProcessDataField(ProcessDefinitionUUID processDefinitionUUID, String dataFieldId)
    throws ProcessNotFoundException, DataFieldNotFoundException {
    return getAPI().getProcessDataField(processDefinitionUUID, dataFieldId);
  }

  public Set<DataFieldDefinition> getProcessDataFields(ProcessDefinitionUUID processDefinitionUUID)
    throws ProcessNotFoundException {
    return getAPI().getProcessDataFields(processDefinitionUUID);
  }

  public Set<DataFieldDefinition> getActivityDataFields(ActivityDefinitionUUID activityDefinitionUUID)
    throws ActivityDefNotFoundException {
    return getAPI().getActivityDataFields(activityDefinitionUUID);
  }

  public DataFieldDefinition getActivityDataField(
    ActivityDefinitionUUID activityDefinitionUUID, String dataFieldId)
    throws ActivityDefNotFoundException, DataFieldNotFoundException {
    return getAPI().getActivityDataField(activityDefinitionUUID, dataFieldId);
  }

}
