package org.ow2.bonita.facade.ejb.ejb2.internal;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.EJBUtil;

public class EJB2InternalQueryRuntimeAPI implements QueryRuntimeAPI {

  protected QueryRuntimeAPI getAPI() {
    return EJBUtil.getEJB2APIAccessor().getQueryRuntimeAPI();
  }
  
  public Set<ActivityInstance<ActivityBody>> getActivityInstances(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getActivityInstances(instanceUUID);
  }

  public Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException {
    return getAPI().getActivityInstances(instanceUUID, activityId);
  }

  public ActivityInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    return getAPI().getActivityInstance(activityUUID);
  }

  public ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getProcessInstance(instanceUUID);
  }

  public Set<ProcessInstance> getProcessInstances() {
    return getAPI().getProcessInstances();
  }
  
  public Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return getAPI().getProcessInstances(processUUID);
  }

  public ActivityInstance<TaskInstance> getTask(TaskUUID taskUUID) throws TaskNotFoundException {
    return getAPI().getTask(taskUUID);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID,
      TaskState taskState) throws InstanceNotFoundException {
    return getAPI().getTaskList(instanceUUID, taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID,
      String userId, TaskState taskState) throws InstanceNotFoundException {
    return getAPI().getTaskList(instanceUUID, userId, taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(TaskState taskState) {
    return getAPI().getTaskList(taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(String userId, TaskState taskState) {
    return getAPI().getTaskList(userId, taskState);
  }

  public Set<ActivityInstance<TaskInstance>> getTasks(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getTasks(instanceUUID);
  }

  public Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId) throws ActivityNotFoundException,
      VariableNotFoundException {
    return getAPI().getActivityInstanceVariable(activityUUID, variableId);
  }

  public Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId) throws InstanceNotFoundException, VariableNotFoundException {
    return getAPI().getProcessInstanceVariable(instanceUUID, variableId);
  }

  public Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    return getAPI().getActivityInstanceVariables(activityUUID);
  }

  public Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getProcessInstanceVariables(instanceUUID);
  }

  public Object getVariable(ActivityInstanceUUID activityUUID, String variableId) 
    throws ActivityNotFoundException, VariableNotFoundException {
    return getAPI().getVariable(activityUUID, variableId);
  }
  public Map<String, Object> getVariables(ActivityInstanceUUID activityUUID) 
    throws InstanceNotFoundException, ActivityNotFoundException {
    return getAPI().getVariables(activityUUID);
  }


}
