package org.ow2.bonita.facade.ejb.ejb2.internal;

import java.util.Map;

import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.EJBUtil;

public class EJB2InternalRuntimeAPI implements RuntimeAPI {

  protected RuntimeAPI getAPI() {
    return EJBUtil.getEJB2APIAccessor().getRuntimeAPI();
  }

  public void startTask(TaskUUID taskUUID, boolean assignTask) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().startTask(taskUUID, assignTask);
  }

  public void finishTask(TaskUUID taskUUID, boolean taskAssign) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().finishTask(taskUUID, taskAssign);
  }

  public void resumeTask(TaskUUID taskUUID, boolean taskAssign) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().resumeTask(taskUUID, taskAssign);
  }

  public void suspendTask(TaskUUID taskUUID, boolean assignTask) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().suspendTask(taskUUID, assignTask);
  }

  public void assignTask(TaskUUID taskUUID) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID);
  }

  public void assignTask(TaskUUID taskUUID, String actorId) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID, actorId);
  }

  public void assignTask(TaskUUID taskUUID, java.util.Set<java.lang.String> candidates) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID, candidates);
  }

  public void deleteAllProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException, 
    UndeletableInstanceException {
    getAPI().deleteAllProcessInstances(processUUID);
  }

  public void deleteProcessInstance(ProcessInstanceUUID instanceUUID) 
    throws InstanceNotFoundException, UndeletableInstanceException {
    getAPI().deleteProcessInstance(instanceUUID);
  }

  public ProcessInstanceUUID instantiateProcess(
      ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return getAPI().instantiateProcess(processUUID);
  }

  public ProcessInstanceUUID instantiateProcess(
      ProcessDefinitionUUID processUUID, Map<String, Object> variables) throws ProcessNotFoundException {
    return getAPI().instantiateProcess(processUUID, variables);
  }

  public void setProcessInstanceVariable(ProcessInstanceUUID instanceUUID, String variableId,
      Object variableValue) throws InstanceNotFoundException, VariableNotFoundException {
    getAPI().setProcessInstanceVariable(instanceUUID, variableId, variableValue);
  }

  public void setActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId, Object variableValue) throws ActivityNotFoundException, VariableNotFoundException {
    getAPI().setActivityInstanceVariable(activityUUID, variableId, variableValue);
  }

  public void setVariable(ActivityInstanceUUID activityUUID, String variableId, Object variableValue) 
    throws ActivityNotFoundException, VariableNotFoundException {
    getAPI().setVariable(activityUUID, variableId, variableValue);
  }

}
