/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import org.jbpm.pvm.env.Environment;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.exception.BonitaWrapperException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.Repository;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.Command;
import org.ow2.novabpm.util.Misc;


public final class FacadeUtil {

  private FacadeUtil() { }
   
  public static <T> T executeCommand(Command<T> command) throws Exception {
    return command.execute(Environment.getCurrent());
  }

  public static XpdlProcess getProcessDefinition(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    Misc.checkArgsNotNull(processUUID);
    Repository repository = EnvTool.getRepository();
    final XpdlProcess process = repository.getXpdlProcess(processUUID);
    if (process == null) {
      throw new ProcessNotFoundException(processUUID);
    }
    return process;
  }

  public static XpdlInstance getInstance(ProcessInstanceUUID instanceUUID) {
    Misc.checkArgsNotNull(instanceUUID);
    // get instance repository
    final Repository repository = EnvTool.getRepository();

    return repository.getXpdlInstance(instanceUUID);
  }

  static void checkArgsNotNull(Object... args) {
    try {
      Misc.checkArgsNotNull(1, args);
    } catch (IllegalArgumentException e) {
      throw new BonitaWrapperException(e);
    }
  }

}


