/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.impl.ActivityInstanceImpl;
import org.ow2.bonita.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.Misc;


/**
 * @author Pierre Vigneras
 */
public class QueryRuntimeAPIImpl implements QueryRuntimeAPI {

  /*
  public QueryRuntimeAPIImpl(String CONF_NAME) {
    this.confName = CONF_NAME;
  }
   */
  protected QueryRuntimeAPIImpl() {
  }
  
  public ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    FacadeUtil.checkArgsNotNull(instanceUUID);
    final ProcessInstance result = EnvTool.getAllQueriers().getProcessInstance(instanceUUID);
    if (result == null) {
      throw new InstanceNotFoundException(instanceUUID);
    }
    return new ProcessInstanceImpl(result);
//  try {
//  return FacadeUtil.executeCommand(new GetProcessInstance(instanceUUID));
//  } catch (InstanceNotFoundException e) {
//  throw e;
//  } catch (RuntimeException e) {
//  throw e;
//  } catch (Exception e) {
//  // This should never happen
//  throw new BonitaInternalException("Ouch! Unexpected error: " + e, e);
//  }
  }

  public Set<ProcessInstance> getProcessInstances() {
    final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
    for (ProcessInstance record : EnvTool.getAllQueriers().getProcessInstances()) {
      result.add(new ProcessInstanceImpl(record));
    }
    return result;
  }
  
  public Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) {
    final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
    for (ProcessInstance record : EnvTool.getAllQueriers().getProcessInstances(processUUID)) {
      result.add(new ProcessInstanceImpl(record));
    }
    return result;
  }

  public ActivityInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, String activityId,
      String iterationId, String activityInstanceId) throws ActivityNotFoundException, InstanceNotFoundException {
    final ActivityInstance<ActivityBody> result =
      EnvTool.getAllQueriers().getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
    if (result == null) {
      if (EnvTool.getAllQueriers().getProcessInstance(instanceUUID) == null) {
        throw new InstanceNotFoundException(instanceUUID);
      }
      throw new ActivityNotFoundException(instanceUUID, activityId, iterationId);
    }
    return new ActivityInstanceImpl<ActivityBody>(result);
  }

  public Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
    final Set<ActivityInstance<ActivityBody>> result = new HashSet<ActivityInstance<ActivityBody>>();
    for (ActivityInstance<ActivityBody> record : EnvTool.getAllQueriers().getActivityInstances(instanceUUID)) {
      result.add(new ActivityInstanceImpl<ActivityBody>(record));
    }
    return result;
  }

  public Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID,
      String activityId) throws ActivityNotFoundException {
    final Set<ActivityInstance<ActivityBody>> result = new HashSet<ActivityInstance<ActivityBody>>();
    for (ActivityInstance<ActivityBody> record : EnvTool.getAllQueriers().getActivityInstances(instanceUUID)) {
      if (record.getActivityId().equals(activityId)) {
        result.add(new ActivityInstanceImpl<ActivityBody>(record));
      }
    }
    if (result.isEmpty()) {
      throw new ActivityNotFoundException(instanceUUID, activityId);
    }
    return result;
  }

  public Set<ActivityInstance< TaskInstance >> getTasks(ProcessInstanceUUID instanceUUID) {
    final Set<ActivityInstance< TaskInstance >> result = new HashSet<ActivityInstance< TaskInstance >>();
    for (ActivityInstance< TaskInstance > record : EnvTool.getAllQueriers().getTaskInstances(instanceUUID)) {
      result.add(new ActivityInstanceImpl< TaskInstance >(record));
    }
    return result;
  }

  public Collection<ActivityInstance< TaskInstance >> getTaskList(final ProcessInstanceUUID instanceUUID,
      TaskState taskState) throws InstanceNotFoundException {
    return getTaskListUser(instanceUUID, EnvTool.getUserId(), taskState);
  }

  public Collection<ActivityInstance< TaskInstance >> getTaskList(TaskState taskState) {
    return getTaskListUser(EnvTool.getUserId(), taskState);
  }

  public ActivityInstance< TaskInstance > getTask(TaskUUID taskUUID) throws TaskNotFoundException {
    final ActivityInstance< TaskInstance > taskInstance = EnvTool.getAllQueriers().getTaskInstance(taskUUID);
    if (taskInstance == null) {
      throw new TaskNotFoundException(taskUUID);
    }
    return new ActivityInstanceImpl< TaskInstance >(taskInstance);
  }

  public Map<String, Object> getActivityInstanceVariables(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId) 
    throws ActivityNotFoundException, InstanceNotFoundException {

    final ActivityInstance<ActivityBody> activityInst = 
      getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
    if (activityInst == null) {
      throw new ActivityNotFoundException(instanceUUID, activityId);
    }
    return activityInst.getLastKnownVariableValues();
  }

  public Object getActivityInstanceVariable(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId, String variableId) throws InstanceNotFoundException,
      ActivityNotFoundException, VariableNotFoundException {
    final Map<String, Object> variables = getActivityInstanceVariables(instanceUUID, activityId, iterationId, activityInstanceId);
    if (variables == null || !variables.containsKey(variableId)) {
      throw new VariableNotFoundException(instanceUUID, activityId, variableId);
    }
    return variables.get(variableId);
  }

  public Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    ProcessInstance processInstance = getProcessInstance(instanceUUID);
    if (processInstance == null) {
      throw new InstanceNotFoundException(instanceUUID);
    }
    return processInstance.getLastKnownVariableValues();
  }

  public Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId) throws InstanceNotFoundException, VariableNotFoundException {
    final Map<String, Object> variables = getProcessInstanceVariables(instanceUUID);
    if (variables == null || !variables.containsKey(variableId)) {
      throw new VariableNotFoundException(instanceUUID, variableId);
    }
    return variables.get(variableId);
  }

/*
  private Map<String, Object> getVariablesInitialValue(List<VariableDefinitionImpl> variableDefinitions) {
    final Map<String, Object> values = new HashMap<String, Object>();
    if (variableDefinitions != null) {
      for (VariableDefinitionImpl variableDefinition : variableDefinitions) {
        final String variableId = variableDefinition.getKey();
        Object initialValue = null;
        if (variableDefinition.getSourceDescriptor() != null) {
          initialValue = variableDefinition.getSourceDescriptor().cget();
        }
        values.put(variableId, initialValue);
      }
    }
    return values;
  }

  @SuppressWarnings("unchecked")
  public Map<String, Object> getActivityVariableDefinitions(ProcessDefinitionUUID processId,
      String activityId) throws ProcessNotFoundException, ActivityNotFoundException {
    final XpdlProcess process  = EnvTool.getRepository().findXpdlProcessById(processId);
    if (process == null) {
      throw new ProcessNotFoundException(processId);
    }
    final NodeImpl node = process.getNode(activityId);
    if (node == null) {
      throw new ActivityNotFoundException(processId, activityId);
    }
    final AbstractActivity activity = (AbstractActivity) node.getBehaviour();
    final List<VariableDefinition> variableDefinitions = new ArrayList<VariableDefinition>();
    if (activity.getVariableDefinitions() != null) {
      variableDefinitions.addAll(activity.getVariableDefinitions());
    }
    if (process.getVariableDefinitions() != null) {
      variableDefinitions.addAll(process.getVariableDefinitions());
    }
    return getVariablesInitialValue((List) variableDefinitions);
  }

  @SuppressWarnings("unchecked")
  public Map<String, Object> getProcessVariableDefinitions(ProcessDefinitionUUID processId) throws ProcessNotFoundException {
    final XpdlProcess process  = EnvTool.getRepository().findXpdlProcessById(processId);
    if (process == null) {
      throw new ProcessNotFoundException(processId);
    }
    final List<VariableDefinition> variableDefinitions = process.getVariableDefinitions();
    return getVariablesInitialValue((List) variableDefinitions);
  }

*/

  public ActivityInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    ActivityInstance<ActivityBody> activity = EnvTool.getAllQueriers().getActivityInstance(activityUUID);
    if (activity == null) {
      throw new ActivityNotFoundException(activityUUID);
    }
    return new ActivityInstanceImpl<ActivityBody>(activity);
  }

  public Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId) throws ActivityNotFoundException,
      VariableNotFoundException {
    ActivityInstance<ActivityBody> activity = getActivityInstance(activityUUID);
    Map<String, Object> variableValues = activity.getLastKnownVariableValues();

    if (!variableValues.containsKey(variableId)) {
      throw new VariableNotFoundException(activityUUID, variableId);
    }
    return activity.getLastKnownVariableValues().get(variableId);
  }

  public Map<String, Object> getActivityInstanceVariables(
      ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    ActivityInstance<ActivityBody> activity = getActivityInstance(activityUUID);
    return activity.getLastKnownVariableValues();
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID,
      String userId, TaskState taskState) throws InstanceNotFoundException {
    return getTaskListUser(instanceUUID, userId, taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(String userId, TaskState taskState) {
    return getTaskListUser(userId, taskState);
  }

  private Collection<ActivityInstance<TaskInstance>> getTaskListUser(ProcessInstanceUUID instanceUUID, String userId,
      TaskState taskState) throws InstanceNotFoundException {
    FacadeUtil.checkArgsNotNull(instanceUUID, taskState, userId);
    ProcessInstance processInstance = getProcessInstance(instanceUUID);
    if (processInstance == null) {
      throw new InstanceNotFoundException(instanceUUID);
    }
    final Querier journal = EnvTool.getAllQueriers();
    final Set<ActivityFullInstance<TaskInstance>> userInstanceTasks =
      journal.getUserInstanceTasks(userId, instanceUUID, taskState);

    final Collection<ActivityInstance<TaskInstance>> todos = new ArrayList<ActivityInstance<TaskInstance>>();
    for (ActivityInstance<TaskInstance> taskActivity : userInstanceTasks) {
      todos.add(new ActivityInstanceImpl<TaskInstance>(taskActivity));
    }
    return todos;
  }
  private Collection<ActivityInstance< TaskInstance >> getTaskListUser(String userId, TaskState taskState) {
    FacadeUtil.checkArgsNotNull(userId, taskState);
    final Collection<ActivityInstance< TaskInstance >> todos = new ArrayList<ActivityInstance< TaskInstance >>();
    final Querier querier = EnvTool.getAllQueriers();
    final Set<ActivityFullInstance<TaskInstance>> taskActivities = querier.getUserTasks(userId, taskState);
    for (ActivityFullInstance<TaskInstance> taskActivity : taskActivities) {
      todos.add(new ActivityInstanceImpl<TaskInstance>(taskActivity));
    }
    return todos;
  }


  public Object getVariable(ActivityInstanceUUID activityUUID, String variableId) 
    throws ActivityNotFoundException, VariableNotFoundException {
    try {
      return getActivityInstanceVariable(activityUUID, variableId);
    } catch (VariableNotFoundException e) {
      ActivityFullInstance<ActivityBody> activity = EnvTool.getAllQueriers().getActivityInstance(activityUUID);
      if (activity == null) {
        throw new ActivityNotFoundException(activityUUID);
      }
      try {
        return getProcessInstanceVariable(activity.getProcessInstanceUUID(), variableId);
      } catch (InstanceNotFoundException e1) {
        // If activity exists, the process instance must exist too.
        Misc.unreachableStatement();
        return null;
      }
    }
  }

  public Map<String, Object> getVariables(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    ActivityFullInstance<ActivityBody> activity = EnvTool.getAllQueriers().getActivityInstance(activityUUID);
    if (activity == null) {
      throw new ActivityNotFoundException(activityUUID);
    }
    try {
      Map<String, Object> allVariables = new HashMap<String, Object>();
      Map<String, Object> localVariables = activity.getLastKnownVariableValues();
      Map<String, Object> globalVariables = getProcessInstanceVariables(activity.getProcessInstanceUUID());
      //add global first because if some variables are in both local and global
      //we want to keep local value
      allVariables.putAll(globalVariables);
      allVariables.putAll(localVariables);
      return allVariables;
    } catch (InstanceNotFoundException e) {
      // If activity exists, the process instance must exist too.
      Misc.unreachableStatement();
      return null;
    }
  }
}
