/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.parsing.binding;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.xml.Parse;
import org.jbpm.pvm.internal.xml.Parser;
import org.ow2.bonita.facade.def.element.ExternalReferenceDefinition;
import org.ow2.bonita.facade.def.element.FormalParameterDefinition;
import org.ow2.bonita.facade.def.majorElement.ApplicationDefinition;
import org.ow2.bonita.facade.def.majorElement.impl.ApplicationDefinitionImpl;
import org.ow2.bonita.facade.uuid.ApplicationDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.w3c.dom.Element;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class ApplicationBinding extends MajorElementBinding {

  private static final Logger LOG = Logger.getLogger(ApplicationBinding.class.getName());

  public ApplicationBinding() {
    super("Application");
  }
  public Object parse(Element applicationElement, Parse parse, Parser parser) {
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("parsing element = " + applicationElement);
    }


    String id = getId(applicationElement);
    String name = getName(applicationElement);

    ApplicationDefinitionUUID recordUUID = new ApplicationDefinitionUUID(
        getObject(PackageDefinitionUUID.class, parse), getObject(ProcessDefinitionUUID.class, parse), name);
    parse.pushObject(recordUUID);

    String description = getChildTextContent(applicationElement, "Description");

    List<FormalParameterDefinition> formalParameters = getFormalParameters(applicationElement, parse, parser);
    if (formalParameters != null && !formalParameters.isEmpty()) {
      parse.addProblem("Formal Parameters not yet supported on Application element.");
    }
    ExternalReferenceDefinition externalReference = getExternalereference(applicationElement);
    if (externalReference != null) {
      parse.addProblem("Externalreference element not yet supported on Application element.");
    }

    ApplicationDefinition applicationDefinition = new ApplicationDefinitionImpl(recordUUID, id,
        getObject(PackageDefinitionUUID.class, parse), getObject(ProcessDefinitionUUID.class, parse),
        description, externalReference, formalParameters, name);

    parse.popObject();
    return applicationDefinition;
  }

}


