package org.ow2.bonita.persistence.db;

import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.def.majorElement.impl.PackageFullDefinitionImpl;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.impl.ProcessFullInstanceImpl;
import org.ow2.bonita.services.Archiver;
import org.ow2.bonita.services.Querier;
import org.ow2.novabpm.util.Misc;

/**
 * 
 * @author Thomas Gueze
 *
 */
public class DbHistory extends AbstractDbQuerier implements Archiver, Querier {

  public DbHistory(String name) {
    super(name, new DbQuerierBufferImpl());
  }
  
  public void archive(PackageFullDefinition packageDef) {
    Misc.checkArgsNotNull(packageDef);
    //if (getPersistenceService().getPackage(packageDef.getPackageDefinitionUUID()) == null) {
    if (getPackage(packageDef.getPackageDefinitionUUID()) == null) {
      PackageFullDefinition pack = new PackageFullDefinitionImpl(packageDef);
      buffer.addPackage(pack);
      getQuerierDbSession().save(pack);
    } else {
      throw new IllegalArgumentException(" Can't archive: " + packageDef
          + " --- A record with processDefinitionUUID: "
          + packageDef + " has already been archived ");   
    }
  }

  public void archive(ProcessFullInstance processInst) {
    Misc.checkArgsNotNull(processInst);
    if (getProcessInstance(processInst.getUUID()) == null) {
      ProcessFullInstance instance = new ProcessFullInstanceImpl(processInst);
      buffer.addInstance(instance);
      getQuerierDbSession().save(instance);
    } else {
      throw new IllegalArgumentException(" Can't archive: " + processInst
          + " --- A record with processInstanceUUID: "
          + processInst + " has already been archived ");
    }
  }
  
  public void remove(PackageFullDefinition package1) {
    Misc.checkArgsNotNull(package1);
    buffer.removePackage(package1);
    getQuerierDbSession().delete(package1);
  }
  
  public void remove(ProcessFullInstance processInst) {
    Misc.checkArgsNotNull(processInst);
    buffer.removeInstance(processInst);
    getQuerierDbSession().delete(processInst);
  }

  public void clear() {
    Set<PackageFullDefinition> packages = getQuerierDbSession().getPackages();
    for (PackageFullDefinition pack : packages) {
      buffer.removePackage(pack);
      getQuerierDbSession().delete(pack);
    }
    Set<ProcessFullDefinition> processes = getQuerierDbSession().getProcesses();
    Set<ProcessFullInstance> processInstances;
    for (ProcessFullDefinition process : processes) {
      processInstances = getQuerierDbSession().getProcessInstances(process.getProcessDefinitionUUID());
      for (ProcessFullInstance processInstance : processInstances) {
        buffer.removeInstance(processInstance);
        getQuerierDbSession().delete(processInstance);
      }
    }
  }
  
}
