/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services;

import org.ow2.bonita.definition.activity.AbstractActivity;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;

/**
 * A Recorder is responsible for storing <strong>runtime</strong> instances
 * such as currently deployed packages and processes, running instances,
 * activities and tasks.
 *
 * Usually, recorded records are available using queries on a {@link Journal}.
 *
 * @author Pierre Vigneras
 * @date Feb 6, 2008
 * @see Journal
 */
public interface Recorder {
  String DEFAULT_KEY = "recorder";

  /*
   * ACTIVITY
   */
  void recordEnterActivity(AbstractActivity activity, XpdlExecution execution);
  void recordBodyStarted(AbstractActivity activity, XpdlExecution execution);
  void recordBodyEnded(AbstractActivity activity, XpdlExecution execution);

  /*
   * INSTANCE
   */
  void recordInstanceStarted(XpdlInstance xpdlInstance, String userId);
  void recordInstanceEnded(XpdlInstance xpdlInstance, String userId);

  /*
   * TASK
   */
  void recordTaskReady(TaskRunTime task);
  void recordTaskStarted(TaskRunTime task, String userId);
  void recordTaskFinished(TaskRunTime task, String userId);
  void recordTaskSuspended(TaskRunTime task, String userId);
  void recordTaskResumed(TaskRunTime task, String userId);
  void recordTaskAssigned(TaskRunTime taskRunTime, String assignerUserId, java.util.Set<java.lang.String> candidates,
      String userId);

  /*
   * DEFINITION
   */
  void recordPackageDeployed(PackageFullDefinition packageDef);
  void recordPackageUndeployed(PackageFullDefinition packageDef, String userId);

  /*
   * VARIABLE UPDATES
   */
  void recordInstanceVariableUpdated(String variableId, Object variableValue, ProcessInstanceUUID instanceUUID, String userId);
  void recordActivityVariableUpdated(String variableId, Object variableValue, ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId, String userId);
  
  /**
   * Generic method for removing a journal record.
   *
   * Implementation may delegate to overloaded methods according to record type.
   * @param record the record to archive
   * @return true if removing was possible, false otherwise.
   */
  void remove(ProcessFullInstance processInst);
  void remove(PackageFullDefinition package1);
}
